#!/bin/bash
#===============================================================================
# Title:       createLogRotateConfig.sh
#
# Disclaimer:  This script is provided "AS IS" and without any official support
#              from Oracle. Its use needs to be performed using details from the
#              comments section and/or readme file (if one is included). Any bugs
#              encountered, feedback, and/or enhancement requests are welcome.
#
#              No liability for the contents of this script can be accepted. Use
#              the concepts, examples, and information at your own risk. However,
#              great care has been taken to ensure that all technical information
#              is accurate and as useful as possible.
#
#
# Description: This script is used to create a logrotate configuration file
#              to be used for a Connectivity Agent installation. The logrotate
#              will manage the amount of stdout retained so as not to exhaust
#              the disk storage on the agent machine. It is similar to the log
#              rotation feature offered by WebLogic server, but is a native
#              Linux command.
#
#              For more details on logrotate see:
#              man logrotate
#              and/or http://www.linuxcommand.org/man_pages/logrotate8.html
#
#              The logrotate configuration file and a shell script are generated
#              in the agent domain directory. The shell script is intended to be
#              used by cron where the crontab entry will look something like:
#
#              */10 *  *  *  * oracle /oracle/agenthome/user_projects/domains/agent-domain/agent_stdout_logrotate.sh
#
#              Details for the crontab are provided as part of the output of
#              createLogRotateConfig.sh
#
# Author:      Greg Mally
#
# Date:        28-March-2017
#
# Version:     1.0    
#
# Usage:       ./createLogRotateConfig.sh -ah=/oracle/agenthome -od=/oracle/agentstdout -op=*.out
#
# Notes:       The above usage assumes that this script has execute permission.
#              If it doesn't have execute permission, then add "bash" in front
#              of the command.  For example, bash ./createLogRotateConfig.sh ...
#
#              It is important to run this script as the agent user since the
#              logrotate configuration file needs to include the user name and
#              default user group as part of the configuration. The user name is
#              also used for the crontab entry.
#
#              Review the Log Rotation Values section for the rotation details.
#              The details include the number of rotation files to keep and the
#              size of the STDOUT file is allowed to grow before it is rotated.
#              The default settings are keep 10 files and rotate the STDOUT file
#              when it grows 1 megabyte or greater in size.
#
#===============================================================================


##################################################
# Log Rotation Values
#
# NOTE: If your logrotate command is not on the
#       $PATH or in some nonstandard location,
#       please change the LOGROTATE value below to
#       match your environment.
#
#       ROTATION_FILE_SIZE can be specified in:
#       k (kilobytes) M (megabytes) G (gigabytes)
#       Valid examples: ROTATION_FILE_SIZE=100k
#                   or: ROTATION_FILE_SIZE=5M
#                   or: ROTATION_FILE_SIZE=1G
##################################################
LOGROTATE=($(which logrotate))
NUMBER_OF_FILES_TO_KEEP=10
ROTATION_FILE_SIZE=1M
AGENT_USER_NAME=$USER
AGENT_USER_GROUP=($(id -Gn))
CONFIG_FILENAME=agent_stdout_logrotate.conf
CRONSCRIPT_FILENAME=agent_stdout_logrotate.sh


##################################################
# Process arguments
##################################################
for i in "$@"
do
case $i in
    -ah=*|--agentHome=*)
    AGENT_HOME="${i#*=}"
    shift
    ;;
    -od=*|--stdoutDirectory=*)
    STDOUT_DIR="${i#*=}"
    shift
    ;;
    -op=*|--stdoutFilePattern=*)
    STDOUT_PATTERN="${i#*=}"
    shift
    ;;
    --help)
    HELP=YES
    ;;
    *)
    # unknown option
    ;;
esac
done


##################################################
# Setup Variables
##################################################
AGENT_DOMAIN=$AGENT_HOME/user_projects/domains/agent-domain

##################################################
# Check for omitted -od and -op arguments to set
# default values for a startAgent.sh command like
# the following:
# nohup ./startAgent.sh -u=... -p=... > $AGENT_DOMAIN/servers/AdminServer/logs/AdminServer.out &
#
# It is assumed that the above command will be
# used when the -od and -op arguments are omitted.
##################################################
if [[ ( ! -z $AGENT_HOME ) && (( -z $STDOUT_DIR ) || ( -z $STDOUT_PATTERN )) ]]; then
    STDOUT_DIR=$AGENT_DOMAIN/servers/AdminServer/logs
    STDOUT_PATTERN=*.out
    echo ""
    echo "Using the following default values based on the startAgent.sh command of"
    echo "nohup $AGENT_HOME/startAgent.sh -u=[ICS USER] -p=[ICS USER PASSWORD] > $STDOUT_DIR/AdminServer.out &"
    echo "-od=$STDOUT_DIR"
    echo "-op=$STDOUT_PATTERN"
    echo ""
fi

STDOUT_LOGFILE=$STDOUT_DIR/$STDOUT_PATTERN
LOGROTATE_CONFIG=$AGENT_DOMAIN/$CONFIG_FILENAME
LOGROTATE_SH=$AGENT_DOMAIN/$CRONSCRIPT_FILENAME


##################################################
# Check for usage
##################################################
if [[ ( -z $AGENT_HOME ) || ( -z $STDOUT_DIR ) || ( -z $STDOUT_PATTERN ) || ( "$HELP" = "YES" ) ]]; then
    echo ""
    echo "usage: $0 -ah=PathToAgenthome [-od=StdoutDirectory] [-op=StdoutFilePattern]"
    echo "For example, $0 -ah=/oracle/agenthome -od=/oracle/agenthome/user_projects/domains/agent-domain/servers/AdminServer/logs -op=*.out"
    echo ""
    echo "NOTE: If the -od and -op arguments are omitted, then something like the following command is assumed to be used to start the agent:"
    echo "      nohup [AGENT_HOME]/startAgent.sh -u=[ICS USER] -p=[ICS USER PASSWORD] > [AGENT_DOMAIN]/servers/AdminServer/logs/AdminServer.out &"
    echo ""
    exit 1
fi


##################################################
# Check for logrotate
##################################################
if [[ -z $LOGROTATE  ]]; then
   LOGROTATE=/usr/sbin/logrotate
fi

if [[ ! -f $LOGROTATE ]]; then
    echo "Unable to locate logrotate, exiting ..."
    exit 1
fi


##################################################
# Check for valid agent home
##################################################
if [[ ! -d $AGENT_HOME ]]; then
    echo "Directory \"$AGENT_HOME\" does not exist, exiting ..."
    exit 1
fi


##################################################
# Check for valid agent domain
##################################################
if [[ ! -d $AGENT_DOMAIN ]]; then
    echo "Directory \"$AGENT_DOMAIN\" does not exist, exiting ..."
    exit 1
fi


##################################################
# Check for valid stdout directory
##################################################
if [[ ! -d $STDOUT_DIR ]]; then
    echo "Directory \"$STDOUT_DIR\" does not exist, exiting ..."
    exit 1
fi


##################################################
# Create logrotate configuration file
##################################################
echo "Creating logrotate configuration file: $LOGROTATE_CONFIG"
echo "# logrotate configuration for Connectivity Agent stdout"                                         >  $LOGROTATE_CONFIG
echo "# For more detaisl see: man logrotate and/or ://www.linuxcommand.org/man_pages/logrotate8.html"  >> $LOGROTATE_CONFIG
echo "$STDOUT_LOGFILE {"                                                                               >> $LOGROTATE_CONFIG
echo "   rotate $NUMBER_OF_FILES_TO_KEEP"                                                              >> $LOGROTATE_CONFIG
echo "   size $ROTATION_FILE_SIZE"                                                                     >> $LOGROTATE_CONFIG
echo "   copytruncate"                                                                                 >> $LOGROTATE_CONFIG
echo "   nocompress"                                                                                   >> $LOGROTATE_CONFIG
echo "   missingok"                                                                                    >> $LOGROTATE_CONFIG
echo "   nodateext"                                                                                    >> $LOGROTATE_CONFIG
echo "   noolddir"                                                                                     >> $LOGROTATE_CONFIG
echo "   create $AGENT_USER_NAME $AGENT_USER_GROUP"                                                    >> $LOGROTATE_CONFIG
echo "}"                                                                                               >> $LOGROTATE_CONFIG


##################################################
# Create logrotate cron script
##################################################
echo "Creating cron script for logrotate:    $LOGROTATE_SH"
echo "#!/bin/sh"                                                                                                        >  $LOGROTATE_SH
echo ""                                                                                                                 >> $LOGROTATE_SH
echo "$LOGROTATE -s /tmp/logrotate.$AGENT_USER_NAME.status.tmp $LOGROTATE_CONFIG"                                       >> $LOGROTATE_SH
echo "RETCODE=\$?"                                                                                                      >> $LOGROTATE_SH
echo "if [ \$RETCODE != 0 ]; then"                                                                                      >> $LOGROTATE_SH
echo "  /usr/bin/logger -t logrotate \"Connectivity Agent STDOUT logrotate finished abnormally with code [\$RETCODE]\"" >> $LOGROTATE_SH
echo "fi"                                                                                                               >> $LOGROTATE_SH
echo "exit 0"                                                                                                           >> $LOGROTATE_SH
chmod +x $LOGROTATE_SH


##################################################
# Provide configuration details
##################################################
echo ""
echo ""
echo "Connectivity Agent logrotate configuration complete for stdout:"
echo $LOGROTATE_CONFIG
cat $LOGROTATE_CONFIG
echo ""
echo "To run logrotate every 10 minutes, add the following line to /etc/crontab on the agent machine:"
echo "*/10 *  *  *  * $AGENT_USER_NAME $LOGROTATE_SH"
echo ""
echo "Then run: sudo service crond reload"
echo ""
echo ""
